// client.cc

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>

#include "Message.h"
#include "ConnectMessage.h"
#include "Int32Message.h"
#include "DLList.h"

int clientInt32Debug = 0;
int clientTimeout = 5;

typedef struct privateData {
    connectStatus connectState;
    int32	valueSent;
    int32	valueReceived;
    SEM_ID	sem;
}privateData;

static void myCallback(Message *message,void *clientPvt)
{
    privateData *pprivateData = (privateData *)clientPvt;
    switch(message->getType()) {
    case messageTypeConnect:
	{
	    ConnectMessage *pConnectMessage = (ConnectMessage *)message;

	    printf("clientInt32 callback messageTypeConnect status %s\n",
		connectStatusString[pConnectMessage->status]);
	    pprivateData->connectState = pConnectMessage->status;
	    delete pConnectMessage;
            semGive(pprivateData->sem);
	}
	break;
    case messageTypeInt32:
	{
            Int32Message *pInt32Message = (Int32Message *)message;
	
	    pprivateData->valueReceived = pInt32Message->value;	
	    if(clientInt32Debug>1) {
		char buffer[100];

		sprintf(buffer,"client value %d pInt32Message %p\n",
		    pprivateData->valueReceived,pInt32Message);
		printf("%s",buffer);
	    }
	    delete pInt32Message;
	    if(pprivateData->valueReceived==pprivateData->valueSent) 
                semGive(pprivateData->sem);
	}
	break;
    default:
	printf("got illegal response message\n");
	break;;
    }
}

int clientInt32(char *server,int slot,int ntoBuffer,int delay)
{
    if(ntoBuffer<1) ntoBuffer = 1;
    privateData	*pprivateData = new privateData;
    pprivateData->connectState = connectNo;
    pprivateData->valueReceived = 0;
    pprivateData->valueSent = 0;
    MessageClient *pMessageClient = new MessageClient(myCallback,pprivateData);
    pprivateData->sem  = semBCreate(SEM_Q_PRIORITY,SEM_EMPTY);
    int status = pMessageClient->bind(server,slot);
    if(status) {
	printf("bind returned %d\n",status);
	return(status);
    }
    while(true) {
	if(pprivateData->connectState!=connectYes) {
	    semTake(pprivateData->sem,60);
	    continue;
	}
	int nwait=0;
	while(pprivateData->valueSent!=pprivateData->valueReceived) {
	    if(nwait==0)printf("client send %d receive %d\n",
		pprivateData->valueSent,pprivateData->valueReceived);
	    //wait an extra second for more messages
	    semTake(pprivateData->sem,60);
	    if(nwait++ == 5) break;
	}
	Int32Message *message=0;
	if(pprivateData->valueSent!=pprivateData->valueReceived) {
	    // never got last message sent. Send it again
	    message = new Int32Message;
	    message->value = pprivateData->valueSent;
	    if(clientInt32Debug>1) printf("sending %d\n",message->value);
	    status = pMessageClient->send(message);
	} else for(int i=0; i<ntoBuffer; i++) {
	    message = new Int32Message;
	    message->value = ++pprivateData->valueSent;
	    if(clientInt32Debug) printf("sending %d\n",message->value);
	    status = pMessageClient->send(message);
	    if(status) break;
	}
	if(status) {
	    printf("send returned %d\n",status);
	    delete message;
	    if(delay==0) taskDelay(1);
	} else {
	    STATUS semStatus = semTake(pprivateData->sem,WAIT_FOREVER);
	    if(semStatus==ERROR) printf("semTake timed out\n");
	}
	if(delay>0) taskDelay(delay);
    }
    return(0);
}
